﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

using G65DDCAPIWrapperSpace;

namespace G65DDCExample
{
    public partial class MainForm : Form
    {
        #region Members

        private Wrapper wrapper = new Wrapper();
        private WaveOutHandler waveOutHandler = new WaveOutHandler();

        private delegate void WrapperDD2PreprocessedStreamCallbackHandler(object sender, DDC2PreprocessedStreamEventArgs eventArgs);
        private delegate void WrapperAudioStreamCallbackHandler(object sender, AudioStreamEventArgs eventArgs);

        #endregion

        #region Constructors

        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        #region Private Methods

        private void InitializeDeviceConnection()
        {
            if (wrapper.Init())
            {
                G65DDCAPIWrapperDeviceInfo aDeviceInfo = wrapper.deviceInfo;

                textBoxSerialNumber.Text = aDeviceInfo.serialNumber;
                textBoxInterface.Text = aDeviceInfo.interfaceType == G65DDCAPIWrapper.DEVICE_INTERFACE_USB3 ? "USB3" :
                                            aDeviceInfo.interfaceType == G65DDCAPIWrapper.DEVICE_INTERFACE_USB2 ? "USB2" : "PCI";

                // Frequency range
                numericUpDownFrequency.Minimum = aDeviceInfo.ranges[0].minFrequency / 1000 /* To kHz */;
                numericUpDownFrequency.Maximum = aDeviceInfo.ranges[1].maxFrequency / 1000 /* To kHz */;
                textBoxFrequencyRange.Text =    numericUpDownFrequency.Minimum + " to " + aDeviceInfo.ranges[0].maxFrequency / 1000 + ", " +
                                                aDeviceInfo.ranges[1].minFrequency / 1000 + " to " + numericUpDownFrequency.Maximum;
                
                numericUpDownFrequency.Value = 87700 /* Example */;

                // Default volume
                trackBarVolume.Value = ((trackBarVolume.Maximum - trackBarVolume.Minimum) / 2) + trackBarVolume.Minimum;
            }
            else
            {
                toolStripStatusLabel.ForeColor = Color.Red;
                if (wrapper.apiStatus == Wrapper.APISTATUS_WRAPPERLIBRARY_NOTFOUND)
                {
                    toolStripStatusLabel.Text = "Unable to find " + Wrapper.APIWRAPPERLIBRARY + " library!";
                }
                else
                {
                    toolStripStatusLabel.Text = "Unable to find device!";
                }
            }

            HandleDeviceConnection();
        }

        private void HandleDeviceConnection()
        {
            if (wrapper.IsDeviceConnected())
            {

                toolStripStatusLabel.ForeColor = Color.Black;
                toolStripStatusLabel.Text = "Device connected";

                panelMain.Enabled = true;

                timerConnection.Start();
            }
            else
            {
                timerConnection.Stop();

                wrapper.CloseDevice();

                toolStripStatusLabel.ForeColor = Color.Red;
                toolStripStatusLabel.Text = "Device not connected";

                textBoxSerialNumber.Text = textBoxInterface.Text = textBoxFrequencyRange.Text = textBoxSignalLevel.Text = "";
                progressBarAudioLevel.Value = 0;
                numericUpDownFrequency.Value = numericUpDownFrequency.Minimum = 0;

                panelMain.Enabled = false;
            }
        }

        #endregion

        #region GUI Event Handlers

        private void OnMainFormLoad(object theSender, EventArgs theEventArgs)
        {
            wrapper.deviceDDC2PreprocessedStreamReceived += new Wrapper.DeviceDDC2PreprocessedStreamReceivedHandler(OnWrapperDDC2PreprocessedStreamReceived);
            wrapper.deviceAudioStreamReceived += new Wrapper.DeviceAudioStreamReceivedHandler(OnWrapperAudioStreamReceived);

            InitializeDeviceConnection();
        }

        private void OnMainFormClosing(object theSender, FormClosingEventArgs theEventArgs)
        {
            timerConnection.Stop();

            if (waveOutHandler.isOpened)
            {
                waveOutHandler.Close();
            }

            if (wrapper.IsDeviceConnected())
            {
                wrapper.CloseDevice();
            }
        }

        private void OnTimerConnectionTick(object theSender, EventArgs theEventArgs)
        {
            HandleDeviceConnection();
        }

        private void OnNumericUpDownFrequencyValueChanged(object theSender, EventArgs theEventArgs)
        {
            if (wrapper.IsDeviceConnected())
            {
                // Set...
                wrapper.SetFrequency((Int32)numericUpDownFrequency.Value * 1000 /* To Hz */);
                // ...and verify
                numericUpDownFrequency.Value = (int)(wrapper.GetFrequency() / 1000 /* To kHz */);

                // Show range
                textBoxRange.Text = wrapper.GetRange() == G65DDCAPIWrapper.RANGE_1 ? "1" : "2";
            }
        }

        private void OnTrackBarVolumeValueChanged(object theSender, EventArgs theEventArgs)
        {
            if (wrapper.IsDeviceConnected())
            {
                // Set...
                wrapper.SetVolume((Byte)trackBarVolume.Value);
                // ...and verify
                trackBarVolume.Value = wrapper.GetVolume();
            }
        }

        #endregion

        #region Wrapper Event Handlers

        private void OnWrapperDDC2PreprocessedStreamReceived(object theSender, EventArgs theEventArgs)
        {
            // If method was called from the same thread, handle received data
            if (!InvokeRequired)
            {
                DDC2PreprocessedStreamEventArgs anEventArgs = (DDC2PreprocessedStreamEventArgs)theEventArgs;

                textBoxSignalLevel.Text = (10.0 * Math.Log10(Math.Pow(anEventArgs.rms, 2) * (1000.0 / 50.0)) /* Conversion do dBm */).ToString("N1") + " dBm";
            }
            // Else, if this method was called from API thread, call it again from the same thread 
            else
            {
                // Prepare calling the method from the same thread
                WrapperDD2PreprocessedStreamCallbackHandler aWrapperHandler = new WrapperDD2PreprocessedStreamCallbackHandler(OnWrapperDDC2PreprocessedStreamReceived);
                object aSender = Thread.CurrentThread;

                BeginInvoke(aWrapperHandler, new object[] { aSender, theEventArgs });
            }
        }

        private void OnWrapperAudioStreamReceived(object theSender, EventArgs theEventArgs)
        {
            if (!waveOutHandler.isOpened)
            {
                return;
            }

            // If method was called from the same thread, handle received data
            if (!InvokeRequired)
            {

                AudioStreamEventArgs anEventArgs = (AudioStreamEventArgs)theEventArgs;

                // Display audio level
                int aValue = 0;
                for (int anIx = 0; anIx < anEventArgs.count; anIx++)
                {
                    aValue = (int)Math.Max(aValue, 100 * Math.Abs(anEventArgs.buffer[anIx]));
                }
                progressBarAudioLevel.Value = aValue < progressBarAudioLevel.Maximum ? aValue : progressBarAudioLevel.Maximum;

                // Write to waveOut
                waveOutHandler.Write(anEventArgs.buffer, anEventArgs.count);
            }
            // Else, if this method was called from API thread, call it again from the same thread 
            else
            {
                // Prepare calling the method from the same thread
                WrapperAudioStreamCallbackHandler aWrapperHandler = new WrapperAudioStreamCallbackHandler(OnWrapperAudioStreamReceived);
                object aSender = Thread.CurrentThread;

                BeginInvoke(aWrapperHandler, new object[] { aSender, theEventArgs });
            }
        }

        #endregion
    }
}
