﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;

using G65DDCAPIWrapperSpace;

namespace G65DDCExample
{
    public class Wrapper
    {
        #region Members

        public G65DDCAPIWrapper apiWrapper = new G65DDCAPIWrapper();
        public G65DDCAPIWrapperDeviceInfo deviceInfo;
        public String deviceSerialNumber;

        public UInt32 apiStatus;

        public static String APIWRAPPERLIBRARY = G65DDCAPIWrapper.APILIBRARY;
        public static UInt32 APISTATUS_WRAPPERLIBRARY_LOADED = G65DDCAPIWrapper.STATUS_APILIBRARY_LOADED;
        public static UInt32 APISTATUS_WRAPPERLIBRARY_NOTFOUND = G65DDCAPIWrapper.STATUS_APILIBRARY_NOT_FOUND;

        public delegate void DeviceDDC2PreprocessedStreamReceivedHandler(object sender, EventArgs eventArgs);
        public event DeviceDDC2PreprocessedStreamReceivedHandler deviceDDC2PreprocessedStreamReceived;
        public delegate void DeviceAudioStreamReceivedHandler(object sender, EventArgs eventArgs);
        public event DeviceAudioStreamReceivedHandler deviceAudioStreamReceived;

        #endregion

        #region Public Methods

        public unsafe bool Init()
        {
            apiStatus = apiWrapper.GetStatus();
            if (apiStatus != APISTATUS_WRAPPERLIBRARY_LOADED)
            {
                return false;
            }

            if (apiWrapper.Enumerate() == 0)
            {
                return false;
            }

            if (apiWrapper.Open() == 0)
            {
                return false;
            }

            deviceInfo = new G65DDCAPIWrapperDeviceInfo();
            deviceInfo.serialNumber = apiWrapper.GetSerialNumber();
            deviceInfo.devicePath = apiWrapper.GetDevicePath();
            deviceInfo.interfaceType = apiWrapper.GetInterfaceType();
            deviceInfo.ranges = apiWrapper.GetRanges();

            // Set callback handlers
            G65DDCAPIWrapper.DDC2PreprocessedStreamCallback ddc2PreprocessedStreamCallback = OnDDC2PreprocessedStreamReceived;
            G65DDCAPIWrapper.AudioStreamCallback audioStreamCallback = OnAudioStreamReceived;
            apiWrapper.RegisterCallbacks(ddc2PreprocessedStreamCallback, audioStreamCallback);

            // Start example streaming.
            // Start DDC1 streaming using ~50ms long buffers.
            UInt32 aDDCTypeIndex;
            UInt32* aDDCTypeIndexPtr = &aDDCTypeIndex;
            G65DDCInfo aDDCInfo = new G65DDCInfo();
            apiWrapper.GetDDC1(aDDCTypeIndexPtr, aDDCInfo); // Get DDC info of DDC1
            apiWrapper.StartDDC1(CalculateSamplesPerBuffer(aDDCInfo.sampleRate, 50 /* Example time in ms */));

            // Start DDC2 streaming using ~50ms long buffers
            apiWrapper.GetDDC2(0 /* Example */, aDDCTypeIndexPtr, aDDCInfo); // Get DDC info of DDC2
            apiWrapper.StartDDC2(0 /* Example */, CalculateSamplesPerBuffer(aDDCInfo.sampleRate, 50 /* Example time in ms */));

            // Start audio streaming using ~50ms long buffers
            apiWrapper.StartAudio(0 /* Example */, CalculateSamplesPerBuffer(48000, 50 /* Example time in ms */));

            return true;
        }

        public unsafe bool IsDeviceConnected()
        {
            int anIsConnected = 0;
            int* anIsConnectedPtr = &anIsConnected;
            apiWrapper.IsConnected(anIsConnectedPtr);

            return anIsConnected != 0;
        }

        public void CloseDevice()
        {
            apiWrapper.StopAudio(0 /* Example */);
            apiWrapper.StopDDC2(0 /* Example */);
            apiWrapper.StopDDC1();

            apiWrapper.Close();
        }

        public void SetFrequency(Int32 theFrequency)
        {
            apiWrapper.SetFrequency(0 /* Example */, theFrequency);
        }

        public unsafe Int32 GetFrequency()
        {
            Int32 aFrequency = 0;
            Int32* aFrequencyPtr = &aFrequency;
            apiWrapper.GetFrequency(0 /* Example */, aFrequencyPtr);

            return aFrequency;
        }

        public unsafe UInt32 GetRange()
        {
            UInt32 aRange = 0;
            UInt32* aRangePtr = &aRange;
            apiWrapper.GetRange(aRangePtr);

            return aRange;
        }

        public void SetVolume(Byte theVolume)
        {
            apiWrapper.SetVolume(0 /* Example */, theVolume);
        }

        public unsafe Byte GetVolume()
        {
            Byte aVolume = 0;
            Byte* aVolumePtr = &aVolume;
            apiWrapper.GetVolume(0 /* Example */, aVolumePtr);

            return aVolume;
        }

        #endregion

        #region Private Methods

        static UInt32 CalculateSamplesPerBuffer(UInt32 theSampleRate, UInt32 theTime)
        {
            UInt32 aSamplesPerBuffer = theTime * theSampleRate / 1000; // Calculte number of I/Q sample-sets per time (in ms) buffer
            return (UInt32)((aSamplesPerBuffer + 63) & (~63)); // Round up to 64
        }

        #endregion

        #region API Callback Handlers

        public unsafe void OnDDC2PreprocessedStreamReceived(UInt32 theChannel, float* theBuffer, UInt32 theCount, float thePeak, float theRms)
        {
            DDC2PreprocessedStreamEventArgs anEventArgs = new DDC2PreprocessedStreamEventArgs();
            anEventArgs.channel = theChannel;
            anEventArgs.count = theCount;
            anEventArgs.peak = thePeak;
            anEventArgs.rms = theRms;
            anEventArgs.buffer = new float[theCount];
            // Copy buffer
            for (int anIx = 0; anIx < theCount; anIx++)
            {
                anEventArgs.buffer[anIx] = theBuffer[anIx];
            }

            // Fire event
            deviceDDC2PreprocessedStreamReceived(this, anEventArgs);
        }

        public unsafe void OnAudioStreamReceived(UInt32 theChannel, UInt32 theStage, float* theBuffer, UInt32 theNumberOfSamples)
        {
            AudioStreamEventArgs anEventArgs = new AudioStreamEventArgs();
            anEventArgs.channel = theChannel;
            anEventArgs.stage = theStage;
            anEventArgs.count = theNumberOfSamples;
            anEventArgs.buffer = new float[2 * theNumberOfSamples];
            // Copy buffer
            for (int anIx = 0; anIx < 2 * theNumberOfSamples; anIx++)
            {
                anEventArgs.buffer[anIx] = theBuffer[anIx];
            }

            // Fire event
            deviceAudioStreamReceived(this, anEventArgs);
        }

        #endregion
    }
}
